/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "entity.h"
#include "ui/uitextbox.h"
#include "locale/language.h"
#include "assert/assert.h"

void npcLoad(entity_t *entity, const entity_t *source) {
  assertNotNull(entity, "Entity pointer cannot be NULL");
  assertNotNull(source, "Source entity pointer cannot be NULL");
  assertTrue(source->type == ENTITY_TYPE_NPC, "Source entity type must be NPC");

  entity->npc = source->npc;
}

void npcUpdate(entity_t *entity) {
  
}

void npcInteract(entity_t *player, entity_t *self) {
  assertTrue(self->type == ENTITY_TYPE_NPC, "Entity must be of type NPC");

  switch(self->npc.interactType) {
    case NPC_INTERACT_TYPE_NONE:
      break;

    case NPC_INTERACT_TYPE_TEXT:
      uiTextboxSetText(languageGet(self->npc.text));
      break;

    case NPC_INTERACT_TYPE_CONVO:
      break;

    case NPC_INTERACT_TYPE_EVENT:
      break;

    default:
      assertUnreachable("Unknown NPC interaction type");
  }

  // uiTextboxSetText(
  //   "Hello World how are you today? Hope things are going well for you! I am "
  //   "having a great day, hope you are too. Did I ever tell you about the tragedy "
  //   "of Darth Plagueis the Wise? He was a dark lord of the Sith, "
  //   "so powerful and so wise he could use the Force to influence the midichlorians"
  // );
  // const char_t *name = "Dom";
  // const char_t *key = "name";
  // uint16_t len = languageFormat(
  //   "test.npc.text",
  //   NULL,
  //   0,
  //   (const char_t *[]){ key },
  //   (const char_t *[]){ name },
  //   1
  // );

  // char_t *buffer = malloc(sizeof(char_t) + len + 1);
  // assertNotNull(buffer, "Failed to allocate buffer for NPC text");

  // languageFormat(
  //   "test.npc.text",
  //   buffer,
  //   len + 1,
  //   (const char_t *[]){ key },
  //   (const char_t *[]){ name },
  //   1
  // );

  // uiTextboxSetText(buffer);
  // free(buffer);
}