/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "drawui.h"
#include "ui/uitextbox.h"
#include "util/memory.h"
#include "assert/assert.h"
#include "display/render.h"

void drawUIInit(void) {
}

void drawUI() {
  drawUITextbox();
}

void drawUITextbox() {
  if(!UI_TEXTBOX.visible) return;

   // Semi-transparent dark blue
  Color background = (Color){ 0, 0, 128, 128 };

  uint32_t x = 0;
  uint32_t y = RENDER_HEIGHT - UI_TEXTBOX_HEIGHT;

  DrawRectangle(
    x, y,
    UI_TEXTBOX_WIDTH, UI_TEXTBOX_HEIGHT,
    background
  );
  
  BeginBlendMode(BLEND_ALPHA);
  if(UI_TEXTBOX.charsRevealed > 0) {
    uint8_t charsRendered = 0;

    // For each line
    for(uint8_t i = 0; i < UI_TEXTBOX_LINES_PER_PAGE; i++) {
      // Get count of chars in the line
      uint8_t lineLength = UI_TEXTBOX.lineLengths[
        i + (UI_TEXTBOX.page * UI_TEXTBOX_LINES_PER_PAGE)
      ];
      if(lineLength == 0) continue;

      // Determine how many chars left to render
      uint8_t lineChars = UI_TEXTBOX.charsRevealed - charsRendered;
      
      // Don't render more than in line
      if(lineChars > lineLength) lineChars = lineLength;
      assertTrue(lineChars > 0, "Line chars must be greater than 0");

      // Update how many rendered
      charsRendered += lineChars;

      for(uint8_t j = 0; j < lineChars; j++) {
        drawUIChar(
          UI_TEXTBOX.text[
            (i * UI_TEXTBOX_CHARS_PER_LINE) + j +
            (UI_TEXTBOX.page * UI_TEXTBOX_CHARS_PER_PAGE)
          ],
          x + UI_TEXTBOX_PADDING_X + UI_TEXTBOX_BORDER_WIDTH + (
            j * FONT_TILE_WIDTH
          ),
          y + UI_TEXTBOX_PADDING_Y + UI_TEXTBOX_BORDER_HEIGHT + (
            i * FONT_TILE_HEIGHT
          ),
          WHITE
        );
      }

      // Check if we're done rendering text
      if(UI_TEXTBOX.charsRevealed - charsRendered == 0) break;
    }
  }
  EndBlendMode();
}

void drawUIText(
  const uint8_t *text,
  const uint16_t length,
  const int32_t x,
  const int32_t y,
  const Color color
) {
  assertNotNull(text, "Text to draw cannot be NULL");
  if(length == 0) return;

  BeginBlendMode(BLEND_ALPHA);
  for(uint16_t i = 0; i < length; i++) {
    drawUIChar(text[i], x + (i * FONT_TILE_WIDTH), y, color);
  }
  EndBlendMode();
}

void drawUIChar(
  const char_t c,
  const int32_t x,
  const int32_t y,
  const Color color
) {
  if(c < ' ' || c > '~') return; // Only print printable ASCII characters

  int32_t charIndex = TILE_INDEXES[c - FONT_CHAR_START];
  int32_t charX = (charIndex % FONT_COLUMN_COUNT) * FONT_TILE_WIDTH;
  int32_t charY = (charIndex / FONT_COLUMN_COUNT) * FONT_TILE_HEIGHT;

  // FLip Y coordinate for raylib texture coordinates
  DrawTextureRec(
    RENDER_FONT_TEXTURE,
    (Rectangle) {
      (float) charX,
      (float) charY, // Flip Y by adding height
      (float) FONT_TILE_WIDTH,
      (float) FONT_TILE_HEIGHT // Negative height to flip the texture
    },
    (Vector2) { x, y }, // Add FONT_HEIGHT to flip Y
    color
  );
}