/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "duskraylib.h"

/**
 * Initializes the UI drawing system.
 */
void drawUIInit(void);

/**
 * Draws all the UI elements to the screen in the correct order.
 */
void drawUI(void);

/**
 * Draws the UI textbox to the screen.
 */
void drawUITextbox(void);

/**
 * Draws text to the screen at the specified position with the given color.
 *
 * @param text The text to draw.
 * @param length The length of the text to draw.
 * @param x The x-coordinate to draw the text at.
 * @param y The y-coordinate to draw the text at.
 * @param color The color of the text.
 */
void drawUIText(
  const uint8_t *text,
  const uint16_t length,
  const int32_t x,
  const int32_t y,
  const Color color
);

/**
 * Draws a single character to the screen at the specified position with the given color.
 *
 * @param c The character to draw.
 * @param x The x-coordinate to draw the character at.
 * @param y The y-coordinate to draw the character at.
 * @param color The color of the character.
 */
void drawUIChar(
  const char_t c,
  const int32_t x,
  const int32_t y,
  const Color color
);